function [eyt, esigt, elogsigt, eunct, valid] = condi_exp_logdiag2017withUNC(A,B,C,PHI,mu,nu,...
    nlength, ... %IRF horizon up to
    nirf, ...    %(nirf)-th shock
    y0,...       % initial value for Y(t)  : (y(-1), y(-2), ..., y(-nlag)
    SIG0,...     % initial value for SIG(t): SIG(-1)
    sig0,...     % vech(SIG0)
    Q, ...       % rotation matrix so that: R = chol(Sig0)'*Q and vbar=R*v
    L, ...       % eliminator 
    D, ...       % duplicator
    Inuwishinv, ... % Wish(I,nu): We reuse to keep the same random seed, dim(InuWish)=ndim*ndim*NN*nlength
    levelshocks) %levelshocks: We reuse to keep the same random seed, dim(levelshocks)=ndim*NN*nlength, standard normal random draws
% where NN: # of simulation
% updates
% 2017/07/04: we use inv(Inuwish) to avoid inverse computation

% %% tester
% nu  = median(gb_nu);
% A   = median(gb_A,3);
% B   = median(gb_B,3);
% C   = median(gb_C,3);
% PHI = median(gb_PHI,3);
% mu  = median(gb_mu,1)';
% 
% nlength = 20;
% nirf    = 1;
% NN      = 10;
% 
% [ndim,nlag] = size(PHI);
% nlag        = nlag/ndim;
% nshock      = ndim*(ndim+1)/2; %number of vol shock
% 
% L = full(EliminationM(ndim));
% D = full(DuplicationM(ndim));
% Atilde = L*kron(A,A)*D;
% Ctilde = L*C(1:end)';
% 
% % starting points: y0, sig0
% Esig = vech2mat((eye(nshock)-Atilde)\Ctilde); %Unconditional variance of innovation
% inmean = fun_mean(Esig);
% PHIsum = zeros(ndim,ndim);
% for ii = 1:nlag
%     PHIsum = PHIsum + PHI(:,ndim*(ii-1)+1:ndim*(ii-1)+ndim);
% end
% 
% y0   = repmat(((eye(ndim)-PHIsum)\mu+(eye(ndim)-PHIsum)\(B*inmean)), 1, nlag);
% SIG0 = Esig;
% sig0 = L*Esig(1:end)';
% 
% % wishart shocks, ndim*ndim*NN*nlength
% I = eye(ndim);
% Inuwish = zeros(ndim,ndim,NN,nlength);
% for j = 1:1:nlength
%     for i = 1:1:NN
%         Inuwish(:,:,i,j) = wishrnd(I,nu,I);
%     end
% end
% 
% % rotation matrix
% [Q,~] = qr(randn(nshock));

%% main file starts here

% dimension
[ndim,nlag] = size(PHI);
nlag        = nlag/ndim;
nshock      = ndim*(ndim+1)/2; %number of vol shock
NN      = size(Inuwishinv,3);
% nlength = size(Inuwishinv,4);

% auxiliary matrices (now defined outside)
% L = full(EliminationM(ndim));
% D = full(DuplicationM(ndim));

% compute IRFs
selector       = zeros(nshock,1);
if nirf>0 %nirf<0, we compute baseline expectation
    selector(nirf) = 1;
end

Sinv = (nu-ndim-1)*(C+A*SIG0*A');
temp_vcvvol  = vcvcalc(Sinv,nu,ndim);
temp_cholvcv = chol(temp_vcvvol, 'lower');
temp_R = temp_cholvcv*Q; %Q is rotation matrix

Atilde = L*kron(A,A)*D;
Ctilde = L*C(1:end)';

Esig1tilde = Ctilde + Atilde*sig0 + temp_R*selector;
Esig1 = reshape(D*Esig1tilde,ndim,ndim);

valid = nan;
% try
%     chol(Esig1);
%     valid = 1;
% catch
%     disp('Esig1 is not positive-definite');
%     valid = 0;
% end

% innov1 = fun_mean(Esig1);
% innov1 = diag(chol((Esig1),'lower'));
innov1 = log(diag(Esig1));

% save matrices
irftemp1    = zeros(ndim,NN,nlength);
irfSIGtemp1 = zeros(ndim,ndim,nlength);
irflogSIGtemp1 = zeros(ndim,ndim,nlength);

% --- ***
irftemp2       = zeros(ndim, NN, nlength); %***to store full process
% ---

% irf on impact
irftemp1(:,:,1)    = repmat( y0(:,1) + B*innov1, 1, NN);
irfSIGtemp1(:,:,1) = Esig1;
irflogSIGtemp1(:,:,1) = diag(log(diag(Esig1)));

% --- ***
% levelshocks = randn(ndim,NN,nlength); %***
irftemp2(:,:,1)      = repmat( y0(:,1) + B*innov1, 1, NN) + chol(Esig1,'lower')*levelshocks(:,:,1); %***
% ---

% irfs after "on impact"
SIG1 = repmat(Esig1, [1,1,NN]);
SIG1diag = zeros(ndim,NN);
for i=2:nlength %irf horizon
    
    lagselector1 = (i-1):-1:1;
    lagselector2 = (i-1):-1:(i-nlag);
    
    for iii = 1:1:NN %simulation
        
        % y(t) for t-1, t-2, ..., t-nlag
        if i<= nlag
            XXX1 = zeros(nlag*ndim,1);
            XXX1(1:(i-1)*ndim,:)  = vec(irftemp1(:,iii,lagselector1));
            XXX1((i-1)*ndim+1:nlag*ndim,:) = vec(y0(:,i:nlag));        

            %--- ***
            XXX2 = zeros(nlag*ndim,1);
            XXX2(1:(i-1)*ndim,:) = vec(irftemp2(:,iii,lagselector1));
            XXX2( (i-1)*ndim+1:nlag*ndim, :) = vec(y0(:,i:nlag));
            %---
        else
            XXX1 = vec(irftemp1(:,iii,lagselector2));
            
            %--- ***
            XXX2 = vec(irftemp2(:,iii,lagselector2));
            %---
        end

        % New (2017/07/04)
        % better way: simulating the current period SV forward
        S1inv = (nu-ndim-1)*(C+A*SIG1(:,:,iii)*A');
        cholS1inv = chol(S1inv, 'lower');
%         SIG1(:,:,iii) = cholS1inv'*Inuwishinv(:,:,iii,i)*cholS1inv;
        SIG1(:,:,iii) = cholS1inv*Inuwishinv(:,:,iii,i)*cholS1inv'; %8-15-17 Molin Edit
        
        temp_logdiagSIG1 = log(diag(SIG1(:,:,iii))); 
        SIG1diag(:,iii) = temp_logdiagSIG1;
 
        %inmean1 = fun_mean(SIG1(:,:,iii));
        %inmean1 = diag(chol(SIG1(:,:,iii),'lower'));
        inmean1 = temp_logdiagSIG1;
                
        irftemp1(:,iii,i) = mu + B*inmean1 + PHI*XXX1;
        
        %--- ***
        irftemp2(:,iii,i) = mu + B*inmean1 + PHI*XXX2 + chol(SIG1(:,:,iii),'lower')*levelshocks(:,iii,i);
        %---
        
    end %end of simulation (NN)
    irfSIGtemp1(:,:,i) = mean(SIG1,3);
    irflogSIGtemp1(:,:,i) = diag(mean((SIG1diag),2));
end %end of horizon (nlength)

% conditional expectations
eyt   = squeeze(mean(irftemp1,2));
esigt = irfSIGtemp1;
elogsigt = irflogSIGtemp1;

%--- ***
eunct = squeeze(var(irftemp2,[],2));
%--- 










