function [A_old, oinfo, mhtune] = draw_A_new(SIG_old, SIGinv_old, nu_old, A_old, C_old, prior, mhtune, L, D)

[ndim, ndim, nobs] = size(SIG_old);

%------
% drawing A

% old draws
%     SIG_old = p_sm_old;
%     SIGinv_old = p_sminv_old;

nshock = ndim*(ndim+1)/2;
Ctilde_old  = L*C_old(1:end)';
Atilde_old  = L*kron(A_old, A_old)*D;
Vinv0_old   = (prior.SIG0.v-ndim-1)*(vech2mat((eye(nshock)-Atilde_old)\Ctilde_old));

% loop
temp_rejA = zeros(1,ndim^2);
temp_mhratioA = zeros(1,ndim^2);
for jj=1:ndim^2
    
    % a) individual updating
    A_prop = A_old;
    temp_omega = sqrt(exp(mhtune.A.logvar(jj,1)))*randn;
    A_prop(jj) = A_old(jj) + temp_omega;
    
    % stability check
    Atilde_prop = L*kron(A_prop, A_prop)*D;
    Vinv0_prop  = (prior.SIG0.v-ndim-1)*(vech2mat((eye(nshock)-Atilde_prop)\Ctilde_old));
    if max(abs(eig(Atilde_prop))) > 1
        temp_AR = 0;
    else
        
        % b) evaluation of Mh ratio
        logAR_up = 0;
        logAR_down = 0;
        
        % prior
        logAR_up = logAR_up + sum(sum(lognormpdf(A_prop, prior.A.m, prior.A.sd)));
        logAR_down = logAR_down + sum(sum(lognormpdf(A_old, prior.A.m, prior.A.sd)));
        
        % t=1 likelihood
        logAR_up = logAR_up + prior.SIG0.v/2*log(det(Vinv0_prop)) ...
            + (prior.SIG0.v+ndim+1)/2*log(det(SIGinv_old(:,:,1))) ...
            - 1/2*trace(Vinv0_prop*SIGinv_old(:,:,1));
        
        logAR_down = logAR_down + prior.SIG0.v/2*log(det(Vinv0_old)) ...
            + (prior.SIG0.v+ndim+1)/2*log(det(SIGinv_old(:,:,1))) ...
            - 1/2*trace(Vinv0_old*SIGinv_old(:,:,1));
        
        % t>1, likelihood
        log_det_SIG_diff = 0;
        log_tr_SIG_diff  = 0;
        for t=2:1:nobs
            
            % Sinv values
            Sinv0_prop = ( (nu_old-ndim-1)*(C_old + A_prop*SIG_old(:,:,t-1)*A_prop') );
            Sinv0_old  = ( (nu_old-ndim-1)*(C_old + A_old*SIG_old(:,:,t-1)*A_old') );
            
            % Det differences
            %log_det_SIG_diff = log_det_SIG_diff ...
            %    + (nu_old/2)*log(det(Sinv0_prop)) + (nu_old+ndim+1)/2*log(det(SIGinv_old(:,:,t))) ...
            %    - (nu_old/2)*log(det(Sinv0_old))  - (nu_old+ndim+1)/2*log(det(SIGinv_old(:,:,t)));
            log_det_SIG_diff = log_det_SIG_diff ...
                + (nu_old/2)*log(det(Sinv0_prop))  ...
                - (nu_old/2)*log(det(Sinv0_old))  ;
            
            
            % trace differences
            %log_tr_SIG_diff = log_tr_SIG_diff ...
            %    -1/2*trace(Sinv0_prop*SIGinv_old(:,:,t)) ...
            %    +1/2*trace(Sinv0_old*SIGinv_old(:,:,t));
            
            log_tr_SIG_diff = log_tr_SIG_diff ...
                -1/2*trace( (Sinv0_prop-Sinv0_old)*SIGinv_old(:,:,t));
            
        end
        % summing up
        temp_AR = logAR_up - logAR_down ...
            +log_det_SIG_diff + log_tr_SIG_diff;
        temp_AR = exp(temp_AR);
    end
    
    % c) Accept/reject decision
    if jj==1 % for identification
        if A_prop(1,1) <0
            temp_AR = 0;
        end
    end
    
    if rand < temp_AR
        A_old = A_prop;
        temp_rejA(jj) = 0;
    else
        temp_rejA(jj) = 1;
    end
    temp_mhratioA(jj) = min(temp_AR,1);
end

% d) output variables
oinfo.rejA = temp_rejA;
oinfo.mhratio = temp_mhratioA;
