function [C_old, oinfo, mhtune] = draw_C_new(SIG_old, SIGinv_old, nu_old, A_old, C_old, prior, mhtune, L, D)

%------
    % drawing C
    [ndim, ndim, nobs] = size(SIG_old);
    nshock = ndim*(ndim+1)/2;
    
    Atilde_old = L*kron(A_old, A_old)*D;
    
    Ctilde_old = L*C_old(1:end)';
    Vinv0_old  = (prior.SIG0.v-ndim-1)*(vech2mat((eye(nshock)-Atilde_old)\Ctilde_old));
    
    % loop
    jj = 0;
    temp_rejC = zeros(1,ndim*(ndim+1)/2);
    temp_mhratioC = zeros(1, ndim*(ndim+1)/2);
    for j0=1:ndim
        for i0=(j0):1:ndim
            
            jj = jj+1;
            
            % a) individual updating
            Cchol_old = chol(C_old,'lower');
            Cchol_prop = Cchol_old;
            
            if i0==j0 % diagonal
                Cchol_prop(i0,j0) = exp(log(Cchol_old(i0,j0)) + sqrt(exp(mhtune.C.logvar(jj)))*randn);
            else %non-diagonal
                Cchol_prop(i0,j0) = Cchol_old(i0,j0) + sqrt(exp(mhtune.C.logvar(jj)))*randn;
            end
            % transform back
            C_prop = Cchol_prop*Cchol_prop';
            
            % b) evaluating MH ratio
            logAR_up = 0;
            logAR_down = 0;
            
            % --- correct sign
            % Jacobian of cholesky decomposition
            detJaco_prop = 2^ndim * prod(diag(Cchol_prop).^(ndim+1-(1:1:ndim)'));
            detJaco_old  = 2^ndim * prod(diag(Cchol_old).^(ndim+1-(1:1:ndim)'));
            
            logAR_up   = logAR_up + log(detJaco_prop);
            logAR_down = logAR_down + log(detJaco_old);
            
            % Jacobian of log of diagonal elements
            logAR_up = logAR_up + sum(log(diag(Cchol_prop)));
            logAR_down = logAR_down + sum(log(diag(Cchol_old)));
            
%             % --- wrong sign
%             % Jacobian of cholesky decomposition
%             detJaco_prop = 2^ndim * prod(diag(Cchol_prop).^(ndim+1-(1:1:ndim)'));
%             detJaco_old  = 2^ndim * prod(diag(Cchol_old).^(ndim+1-(1:1:ndim)'));
%             
%             logAR_up   = logAR_up - log(detJaco_prop);
%             logAR_down = logAR_down - log(detJaco_old);
%             
%             % Jacobian of log of diagonal elements
%             logAR_up = logAR_up - sum(log(diag(Cchol_prop)));
%             logAR_down = logAR_down - sum(log(diag(Cchol_old)));
%             % ---
            
            
            % prior (inv-Wishart)
            logAR_up = logAR_up - (prior.C.df+ndim+1)/2*log(det(C_prop)) - 1/2*trace(prior.C.C/C_prop);
            logAR_down = logAR_down - (prior.C.df+ndim+1)/2*log(det(C_old)) - 1/2*trace(prior.C.C/C_old);

            % t=1 likelihood
            Ctilde_prop = L*C_prop(1:end)';
            Vinv0_prop  = (prior.SIG0.v-ndim-1)*(vech2mat((eye(nshock)-Atilde_old)\Ctilde_prop));
            
            logAR_up = logAR_up + prior.SIG0.v/2*log(det(Vinv0_prop)) ...
                + (prior.SIG0.v+ndim+1)/2*log(det(SIGinv_old(:,:,1))) ...
                - 1/2*trace(Vinv0_prop*SIGinv_old(:,:,1));
            
            logAR_down = logAR_down + prior.SIG0.v/2*log(det(Vinv0_old)) ...
                + (prior.SIG0.v+ndim+1)/2*log(det(SIGinv_old(:,:,1))) ...
                - 1/2*trace(Vinv0_old*SIGinv_old(:,:,1));
            
            % t>1 likelihood
            log_det_SIG_diff = 0;
            log_tr_SIG_diff  = 0;
            for t=2:1:nobs
                
                % Sinv values
                ASA_old = A_old*SIG_old(:,:,t-1)*A_old';
                Sinv_prop = ( (nu_old-ndim-1)*(C_prop + ASA_old) );
                Sinv_old  = ( (nu_old-ndim-1)*(C_old + ASA_old) );
                
                % Det differences
                %log_det_SIG_diff = log_det_SIG_diff ...
                %    + (nu_old/2)*log(det(Sinv_prop)) + (nu_old+ndim+1)/2*log(det(SIGinv_old(:,:,t))) ...
                %    - (nu_old/2)*log(det(Sinv_old))  - (nu_old+ndim+1)/2*log(det(SIGinv_old(:,:,t)));
%               % The last term can be canceled out ...
                log_det_SIG_diff = log_det_SIG_diff ...
                    + (nu_old/2)*log(det(Sinv_prop))  ...
                    - (nu_old/2)*log(det(Sinv_old));
                
                % trace differences
                %log_tr_SIG_diff = log_tr_SIG_diff ...
                %    -1/2*trace(Sinv_prop*SIGinv_old(:,:,t)) ...
                %    +1/2*trace(Sinv_old*SIGinv_old(:,:,t));
                
                log_tr_SIG_diff = log_tr_SIG_diff ...
                    -1/2*trace( (Sinv_prop-Sinv_old)*SIGinv_old(:,:,t)) ...
                    ;
                
            end
            temp_AR = logAR_up-logAR_down ...
                + log_det_SIG_diff + log_tr_SIG_diff;
            temp_AR = exp(temp_AR);
            
            % c) Accept/reject decision
            if rand < temp_AR % accept
                C_old = C_prop;
                temp_rejC(jj) = 0;
            else
                temp_rejC(jj) = 1;
            end
            temp_mhratioC(jj) = min(temp_AR,1);
        end
    end
    % d) output variabes
    oinfo.rejC = temp_rejC;
    oinfo.mhratio = temp_mhratioC;
    
