function fun_varcaiw2_r(gbrun)

% for CAIW-VAR for HPCC 

is_office = 2; %0=Matlab2016 (laptop), 1=Matlab2017a (office), 2=HPCC

%% Path control
workpath = pwd;
savepath = [workpath, filesep, 'results_r'];
chk_dir(savepath);

%% Estimation option

estopt.level = 1; %1: do level, 0: do diff
estopt.prior = 0; % "0" is prior=11 in the previous code
estopt.pnum = 1;  %1=Molin's most recent prior, 2=Minchul's arbitrary prior

estopt.sam   = 1; %sam=1, starts from 1973M1, sam=2 does not include ZLB, sam=3 after great moderation
estopt.data  = 1;
%0: two variable (ip and ebp)
%1: epb, 2: spread, 3: ebp + govt debt, 4: spread + govt debt,...
%5: expanded data set with ebp, 6: simulated data from GK model 7:
%quarterly spread and TFP ONLY, 8: quarterly spread and Inv TFP ONLY
dataname = ['d', num2str(estopt.data)];

estopt.gbrun = gbrun; %mcmc-seq-index, 1: initial attempt(y(97)), 2:Full sample, 3:shorter sample
estopt.seed  = rng('shuffle');

estopt.np      = 100; % number of particles
estopt.nsimul  = 200000;
estopt.nsave1  = 20; %save every nsave (thinning)
estopt.nsave2  = 200; %thinning for volatility
estopt.nreport = 1000; % for every nreport, we print out results
estopt.nadapt  = 1000; %number of initial simulations for the adaptation

% dimension info
nlag = 12;
estopt.nlag = nlag; %VAR lag order
estopt.f    = 1;    %log(diag), 2 diag(chol)
% estopt.ldim = 2;    %dim(B) = ndim*ldim
% estopt.ndim = 2;    % number of variables 

specname = ['pg_lev',num2str(estopt.level), ...
    'f',num2str(estopt.f), 'p',num2str(estopt.prior), 'sam', num2str(estopt.sam), ...
    dataname, 'lag', num2str(estopt.nlag), 'gbrun',num2str(estopt.gbrun)];
estopt.specname = specname;


%% Load data and Pre-estimation analysis
% load('data_ng');
load('data_ng2')
% data ordering: {'ipm','manemp','hoursm','cpi','wage','ffr','stock','volatbl','sp'}
% we need
% [ip, infl, ffr, sp]
if estopt.data == 5
    y = data_ng(:,[1,3,4,5,6,7,9]);
else
    y = data_ng(:,[1,4,6,9]);
end

% 2-variable VAR
if estopt.data == 0 %baseline two variable
    load('gzebp_mfac.mat');
    y = [y(:,1), data(1:end,2)];
    % transformation
    if estopt.level == 0
        y = [diff(log(y(:,1))*100), y(2:end,2)];
        xcal = xcal(2:end,1);
    elseif estopt.level == 1
        y(:,1) = log(y(:,1))*100;
    end
elseif estopt.data == 1 %baseline four variable
    load('gzebp_mfac.mat');
    y(:,4) = data(1:end,2);
    
    % transformation
    if estopt.level == 0
        y = [diff(log(y(:,1:2))*100), y(2:end,3:4)];
        xcal = xcal(2:end,1);
    elseif estopt.level == 1
        y(:,1:2) = log(y(:,1:2))*100; %log IP and Price
    end
end

% sample selection
if estopt.sam == 1 %full sample
elseif estopt.sam == 2 %before ZLB
    y    = y(1:430,:);
    xcal = xcal(1:430,:);
    % y = y(1:end,:); %97=1981, or 121=1983
elseif estopt.sam == 3 %after great moderation
    selind = xcal>=1984;
    y = y(selind,:);
    xcal = xcal(selind,:);
end

% for prior 
if estopt.sam == 1
    load stdshocksdata1-2
elseif estopt.sam == 2
    load stdshocksdata1-2nozlb
else
    load stdshocksdata1-2
end

if estopt.data == 0
    estopt.stdshocks = stdshocks([1,4]);
elseif estopt.data == 1
    estopt.stdshocks = stdshocks;
end

%% construct VAR variable
% YYY = ytoday
% XXX = [ylag(1), ylag(2), ..., ylag(nlag)];
YYY = y(nlag+1:end,:);
XXX = [];
for i=1:1:nlag
    XXX = [XXX, y(nlag+1-i:end-i,:)];
end
[nobs,ndim] = size(YYY);

estopt.nobs = nobs;
estopt.ndim = ndim;
estopt.ldim = ndim;

X00    = [ones(size(YYY,1),1), XXX];
Bols   = (X00'*X00)\(X00'*YYY);
errors = YYY-X00*Bols;
estopt.VARerrors = errors;

%% Prior
if estopt.prior == 0
    [prior, mhtune] = setup_prior0(estopt);  
end

%% run code
script_CAIW_mexr;

%% Save results
cd(savepath);
savefilename = [estopt.specname, '.mat'];
save(savefilename);
cd(workpath);
