%% new irf main file (minchul, 2017/12/19)
% production version
clc; clear all; close all;
clear -mex;

addpath('HalfVectorization');
addpath('data');
addpath('prior');

% mex function setting
% is_mex = 0: no mex file
% is_mex = 1: matlab 2017
% is_mex = 2: matlab 2016a
is_mex = 2;

is_shortirf = 1;
% 1 if short seq just to check whether the program runs (quick but inaccurate)
% otherwise, the program generates many rotation matrices (slow but accurate)

% try 1: NN=200, nQ=1000, draw00=1000
% try 2: NN=1000, nQ=30, draw00=4000

% setenv('MW_MINGW64_LOC','C:\TDM-GCC-64')

%is_office = 1; %0=No mex, 2=Matlab2016 (laptop), 1=Matlab2017a (office), 2=HPCC
%if is_office == 2
%    addpath('cholinv');
%elseif is_office == 1
%    addpath('cholinv_office');
%end

%% Load settng

% original
%convcheck.m = 10;
%convcheck.nburn = 1000;

% production version (just to check ...)
convcheck.m = 1;
convcheck.nburn = 10;

%% Path control
convcheck.workpath = pwd;
convcheck.loadpath = [convcheck.workpath, filesep, 'results_r'];
convcheck.savepath = convcheck.loadpath;

%% Load files
big_nu = [];
big_A  = [];
big_C  = [];
big_mu  = [];
big_B  = [];
big_PHI  = [];
for temp_gbrun = 1:1:convcheck.m
    % ----------------------------
    % Estimation option
    
    estopt.level = 1; %1: do level, 0: do diff
    estopt.prior = 0; % "0" is prior=11 in the previous code
    estopt.pnum = 1;  %1=Molin's most recent prior, 2=Minchul's arbitrary prior
    
    estopt.sam   = 1; %sam=1, starts from 1973M1, sam=2 does not include ZLB, sam=3 after great moderation
    estopt.data  = 1;
    dataname = ['d', num2str(estopt.data)];
    
    estopt.gbrun = temp_gbrun; %mcmc-seq-index, 1: initial attempt(y(97)), 2:Full sample, 3:shorter sample
    
    % dimension info
    nlag = 12;
    estopt.nlag = nlag; %VAR lag order
    estopt.f    = 1;    %log(diag), 2 diag(chol)
    
    specname = ['pg_lev',num2str(estopt.level), ...
        'f',num2str(estopt.f), 'p',num2str(estopt.prior), 'sam', num2str(estopt.sam), ...
        dataname, 'lag', num2str(estopt.nlag), 'gbrun',num2str(estopt.gbrun)];
    estopt.specname = specname;
    
    % ------------------------------
    % load data
    loadfilename = [estopt.specname, '.mat'];
    cd(convcheck.loadpath);
    load(loadfilename);
    cd(convcheck.workpath);
    
    % collect results
    big_nu = cat(1,big_nu, gb_nu(convcheck.nburn+1:end,:));
    big_A  = cat(3,big_A, gb_A(:,:,convcheck.nburn+1:end));
    big_C  = cat(3,big_C, gb_C(:,:,convcheck.nburn+1:end));
    
    big_mu  = cat(1,big_mu, gb_mu(convcheck.nburn+1:end,:));
    big_B  = cat(3,big_B, gb_B(:,:,convcheck.nburn+1:end));
    big_PHI  = cat(3,big_PHI, gb_PHI(:,:,convcheck.nburn+1:end));
end

% big_A   = permute(big_A, [3,4,1,2]);
% big_C   = permute(big_C, [3,4,1,2]);
% big_mu  = permute(big_mu, [1,3,2]);
% big_B   = permute(big_B, [3,4,1,2]);
% big_PHI = permute(big_PHI, [3,4,1,2]);

%% Preparation

% mex function setting
% is_mex = 0: no mex file
% is_mex = 1: matlab 2017
% is_mex = 2: matlab 2016a
%is_mex = 0; % I move this to the beginning of the code
if is_mex == 0
    my_Inuwishinv = @gen_Inuwishinv;
    my_condexp = @condi_exp_logdiag2017withUNC; %no mex
elseif is_mex == 1
    my_Inuwishinv = @gen_Inuwishinv_mex;
    my_condexp = @condi_exp_logdiag2017withUNC_mex; %matlab2017
elseif is_mex == 2
    my_Inuwishinv = @gen_Inuwishinv_mex_Molin;
    my_condexp = @condi_exp_logdiag2017withUNC_mex_Molin; %matlab2016a
end

% simulation setting

if is_shortirf == 1 %short version
    NN      = 200; %number of simulations
    nlength = 40;  %irf length
    KK      = 1;   %sign restrictions hold for the first KK periods
    nQ      = 100; %number of Q trials per draw
    ndraw00 = 20; %number of draws to compute objects of interest
else %original version
    NN      = 200; %number of simulations
    nlength = 40;  %irf length
    KK      = 1;   %sign restrictions hold for the first KK periods
    nQ      = 1000; %number of Q trials per draw
    ndraw00 = 1000; %number of draws to compute objects of interest
end


% auxiliary variable
L = full(EliminationM(ndim));
D = full(DuplicationM(ndim));

% dimension information
SIGlag = 1;
[ndim, trash, totaldraw] = size(big_PHI);
nlag = trash/ndim;

ngap   = floor(totaldraw/ndraw00);
nshock = ndim*(ndim+1)/2;
nshockcompute = 2; %we compute only two shocks

% random shocks
levelshocks = randn(ndim,NN,nlength); %level shocks
snr = randn(ndim*(ndim-1)/2,NN,nlength); %for wishart shocks

% matrices to store
big_irf   = zeros(ndim, nlength, nshockcompute, nQ*ndraw00);
big_sirf  = zeros(ndim, nlength, nshockcompute, nQ*ndraw00);
big_sLirf = zeros(ndim, nlength, nshockcompute, nQ*ndraw00);
big_sUirf = zeros(ndim, nlength, nshockcompute, nQ*ndraw00);
big_rest  = zeros(nQ*ndraw00,1);

big_irf_max = zeros(ndim,nlength,nshockcompute,ndraw00);
big_irf_min = zeros(ndim,nlength,nshockcompute,ndraw00);
big_irf_valid_minmax = zeros(ndraw00,1);

%% Actual computation
disp('computing irfs ...');
nreport = 10;
irftic = tic;

% iterations for draws
for drawindx = 1:1:ndraw00; % for drawindx = 1:1:ndraw00
    
    sind = (drawindx-1)*ngap+1;
    
    PHI = big_PHI(:,:,sind);
    mu  = big_mu(sind,:)';
    nu  = big_nu(sind,:);
    A   = big_A(:,:,sind);
    B   = big_B(:,:,sind);
    C   = big_C(:,:,sind);
    
    [ndim,nlag] = size(PHI);
    nlag = nlag/ndim;
    nshock = ndim*(ndim+1)/2; %number of vol shock
    Atilde = L*kron(A,A)*D;
    Ctilde = L*C(1:end)';
    
    % wishart shocks, ndim*ndim*NN*nlength
    % old
    % I = eye(ndim);
    % Inuwish = zeros(ndim,ndim,NN,nlength);
    % Inuwishinv00 = zeros(ndim,ndim,NN,nlength);
    % for j=1:1:nlength
    %     for i=1:1:NN
    %         Inuwish(:,:,i,j) = wishrnd(I,nu,I);
    %         Inuwishinv00(:,:,i,j) = inv(Inuwish(:,:,i,j));
    %     end
    % end
    %
    c2r = chi2rnd( repmat( (nu-(0:ndim-1))', [1,NN, nlength]) );
    Inuwishinv = my_Inuwishinv(c2r,snr);
    
    % starting points: y0, sig0
    Sigstart = vech2mat( (eye(nshock)-Atilde)\Ctilde ); %Unconditional variance of innovation
    % We now want to integarte out the volatility, so we simulate to an unc
    for ssim = 1:1000
        Sigstart = iwishrnd( (A*Sigstart*A'+C)*(nu-ndim-1), nu);
    end
    Esig = Sigstart;
    inmean = log(diag(Esig));
    PHIsum = zeros(ndim,ndim);
    for ii=1:nlag
        PHIsum = PHIsum + PHI(:,ndim*(ii-1)+1:ndim*(ii-1)+ndim);
    end
    
    y0 = repmat( ( (eye(ndim)-PHIsum)\mu + (eye(ndim)-PHIsum)\(B*inmean)), 1, nlag);
    SIG0 = Esig;
    sig0 = L*Esig(1:end)';
    
    % -------------------
    % actual IRF computation
    % loop over rotation matrix
    temp_big_irf   = zeros(ndim,nlength,nshockcompute,nQ);
    temp_big_sirf  = zeros(ndim,nlength,nshockcompute,nQ);
    temp_big_sLirf = zeros(ndim,nlength,nshockcompute,nQ);
    temp_big_sUirf = zeros(ndim,nlength,nshockcompute,nQ);
    temp_Q = zeros(nshock,nshock,nQ);
    temp_rest = zeros(nQ,1); %restrictions

    %---------------------------------
    % Compute the baseline expectation
    [eyt0, esigt0, elogsigt0, eunct0, valid] = my_condexp(A,B,C,PHI,mu,nu,...
        nlength, ... %IRF horizon up to
        0, ...    %(nirf)-th shock
        y0,...       % initial value for Y(t)  : (y(-1), y(-2), ..., y(-nlag)
        SIG0,...     % initial value for SIG(t): SIG(-1)
        sig0,...     % vech(SIG0)
        eye(nshock), ...       % rotation matrix so that: R = chol(Sig0)'*Q and vbar=R*v
        L, ...       % eliminator
        D, ...       % duplicator
        Inuwishinv, ... % Wish(I,nu): We reuse to keep the same random seed, dim(InuWish)=ndim*ndim*NN*nlength
        levelshocks); %levelshocks: We reuse to keep the same random seed, dim(levelshocks)=ndim*NN*nlength, standard normal random draws
    
    for Qind = 1:1:nQ; %iteration for Q, Qind=1:1:nQ
        
        
        %---------------------------------
        % compute conditinoal expectation given the rotation
        % rotation matrix
        [Q,RRR] = qr(randn(nshock)); %random rotation
        for i0=1:nshock;
            if RRR(i0,i0)<0 %normalization, See Zha's paper
                Q(:,i0)=-Q(:,i0);
            end
        end
        
        %---------------------------------
        % check restrictions first (only compute KK horizon irfs)
        irf_all   = zeros(ndim,KK,nshock);
        sirf_all  = zeros(ndim,KK,nshock);
        sLirf_all = zeros(ndim,KK,nshock); %log vol
        sUirf_all = zeros(ndim,KK,nshock); %unc
        for shockind = 1:1:nshock; %iteration for shocks
            nirf = shockind;
            
            % conditional expectation
            [eyt1, esigt1, elogsigt1, eunct1, valid] = my_condexp(A,B,C,PHI,mu,nu,...
                KK, ... %IRF horizon up to
                nirf, ...    %(nirf)-th shock
                y0,...       % initial value for Y(t)  : (y(-1), y(-2), ..., y(-nlag)
                SIG0,...     % initial value for SIG(t): SIG(-1)
                sig0,...     % vech(SIG0)
                Q, ...       % rotation matrix so that: R = chol(Sig0)'*Q and vbar=R*v
                L, ...       % eliminator
                D, ...       % duplicator
                Inuwishinv, ... % Wish(I,nu): We reuse to keep the same random seed, dim(InuWish)=ndim*ndim*NN*nlength
                levelshocks); %levelshocks: We reuse to keep the same random seed, dim(levelshocks)=ndim*NN*nlength, standard normal random draws
            
            % mean response
            irf1 = eyt1-eyt0(:,1:KK);
            irf_all(:,:,shockind) = irf1;
            
            % vol response
            sirf1 = esigt1-esigt0(:,:,1:KK);
            for ii=1:1:ndim
                sirf_all(ii,:,shockind) = squeeze(sirf1(ii,ii,:));
            end
            
            % log-vol response
            sLirf1 = elogsigt1-elogsigt0(:,:,1:KK);
            for ii=1:1:ndim
                sLirf_all(ii,:,shockind) = squeeze(sLirf1(ii,ii,:));
            end
            
            % uncertainty response
            sUirf1 = eunct1-eunct0(:,1:KK);
            sUirf_all(:,:,shockind) = sUirf1;
            
        end
        %-----------------
        % check restrictions (baseline)
        find = nan;
        mind = nan;
        shockset1 = (1:1:nshock);
        for sind1 = 1:1:nshock
            rest_all = false;
            
            rest_f1 = sirf_all(1,1,sind1)>0 && sirf_all(2,1,sind1)>0 && sirf_all(3,1,sind1)>0 && sirf_all(4,1,sind1)>0;
            rest_f2 = all(irf_all(4,1:KK,sind1) >0); %func->ebp positive
            
            shockset2 = setdiff( shockset1, sind1);
            if rest_f1&&rest_f2 %crit 1
                for sind2 = shockset2 %crit 2
                    rest_fm = all(irf_all(4,1:KK,sind1) > irf_all(4,1:KK,sind2)); %func->ebp larger than that of munc
                    rest_m1 = sirf_all(1,1,sind2)>0 && sirf_all(2,1,sind2)>0 && sirf_all(3,1,sind2)>0 && sirf_all(4,1,sind2)>0;
                    
                    shockset3 = setdiff( shockset2, sind2);
                    if rest_fm&&rest_m1 %crit 3
                        rest_o = 0; %other shocks do not increase all volatilities
                        for shockind = shockset3
                            rest_o = rest_o + ~(sirf_all(1,KK,shockind)>0 && sirf_all(2,KK,shockind)>0 && sirf_all(3,KK,shockind)>0 && sirf_all(4,KK,shockind)>0);
                        end
                        rest_o = rest_o == (nshock-2);
                        rest_all = rest_f1&&rest_f2&&rest_fm&&rest_m1&&rest_o;
                    end
                    
                    if rest_all
                        break;
                    end
                end
            end
            if rest_all
                find = sind1;
                mind = sind2;
                break;
            end
        end

        
        %---------------------------------
        % Actual IRF computation with full length
        irf_all   = nan*ones(ndim,nlength,nshockcompute);
        sirf_all  = nan*ones(ndim,nlength,nshockcompute);
        sLirf_all = nan*ones(ndim,nlength,nshockcompute); %log vol
        sUirf_all = nan*ones(ndim,nlength,nshockcompute); %unc
                
        if rest_all == 1
            shockindcompute = 1;
            for shockind = [find,mind] %iteration for shocks
                nirf = shockind;
                
                % conditional expectation
                [eyt1, esigt1, elogsigt1, eunct1, valid] = my_condexp(A,B,C,PHI,mu,nu,...
                    nlength, ... %IRF horizon up to
                    nirf, ...    %(nirf)-th shock
                    y0,...       % initial value for Y(t)  : (y(-1), y(-2), ..., y(-nlag)
                    SIG0,...     % initial value for SIG(t): SIG(-1)
                    sig0,...     % vech(SIG0)
                    Q, ...       % rotation matrix so that: R = chol(Sig0)'*Q and vbar=R*v
                    L, ...       % eliminator
                    D, ...       % duplicator
                    Inuwishinv, ... % Wish(I,nu): We reuse to keep the same random seed, dim(InuWish)=ndim*ndim*NN*nlength
                    levelshocks); %levelshocks: We reuse to keep the same random seed, dim(levelshocks)=ndim*NN*nlength, standard normal random draws
                
                % mean response
                irf1 = eyt1-eyt0;
                irf_all(:,:,shockindcompute) = irf1;
                
                % vol response
                sirf1 = esigt1-esigt0;
                for ii=1:1:ndim
                    sirf_all(ii,:,shockindcompute) = squeeze(sirf1(ii,ii,:));
                end
                
                % log-vol response
                sLirf1 = elogsigt1-elogsigt0;
                for ii=1:1:ndim
                    sLirf_all(ii,:,shockindcompute) = squeeze(sLirf1(ii,ii,:));
                end
                
                % uncertainty response
                sUirf1 = eunct1-eunct0;
                sUirf_all(:,:,shockindcompute) = sUirf1;
               
                % +1 to index
                shockindcompute = shockindcompute + 1;
            end
        end
        
        % store
        temp_big_irf(:,:,:,Qind) = irf_all;
        temp_big_sirf(:,:,:,Qind) = sirf_all;
        temp_big_sLirf(:,:,:,Qind) = sLirf_all;
        temp_big_sUirf(:,:,:,Qind) = sUirf_all;
        temp_Q(:,:,Qind) = Q;
        temp_rest(Qind,:) = rest_all;
        
    end %end of nQ
    
    % for prior-robust inference
    if sum(temp_rest)>floor(nQ*0.1)
        temp_QQ = temp_big_irf(:,:,:,logical(temp_rest));
        temp_big_irf_max = max(temp_QQ,[],4);
        temp_big_irf_min = min(temp_QQ,[],4);
        valid_minmax = 1;
    else
        temp_big_irf_max = nan*ones(size(temp_big_irf(:,:,:,1)));
        temp_big_irf_min = nan*ones(size(temp_big_irf(:,:,:,1)));
        valid_minmax = 0;
    end
    
    
    % save
    i0 = (drawindx-1)*nQ+1;
    i1 = drawindx*nQ;
    big_irf(:,:,:,i0:i1)  = temp_big_irf;
    big_sirf(:,:,:,i0:i1) = temp_big_sirf;
    big_sLirf(:,:,:,i0:i1) = temp_big_sLirf;
    big_sUirf(:,:,:,i0:i1) = temp_big_sUirf;
    big_rest(i0:i1,:) = temp_rest;
    
    big_irf_max(:,:,:,drawindx) = temp_big_irf_max;
    big_irf_min(:,:,:,drawindx) = temp_big_irf_min;
    big_irf_valid_minmax(drawindx,1) = valid_minmax;
    
    % report
    if rem(ndraw00,nreport) == 0
        disp(['irf with draw at ', num2str(drawindx), ' / ', num2str(ndraw00)]);
        temp_tt = toc(irftic);
        disp(['remained time : ', num2str(temp_tt*(ndraw00-drawindx+1))]);
        irftic = tic;
    end
    
end %end of drawindx

%% summarizing results
% number of valid draws
sum(big_rest)

big_irf(:,:,:,~logical(big_rest)) = [];
big_sirf(:,:,:,~logical(big_rest)) = [];
big_sLirf(:,:,:,~logical(big_rest)) = [];
big_sUirf(:,:,:,~logical(big_rest)) = [];

cd(convcheck.savepath);
save('irf_com_prod.mat', ...
    'big_irf', 'big_sirf', 'big_sLirf', 'big_sUirf', 'big_rest', ...
    'big_irf_max', 'big_irf_min', 'big_irf_valid_minmax');
cd(convcheck.workpath);

%% Some IRfs...
figure(1)
plot(squeeze(big_irf(1,:,1,:)), 'b')
hold on
plot(squeeze(big_irf(1,:,2,:)), 'r')

plot(median(squeeze(big_irf(1,:,1,:)),2), 'k', 'linewidth',3)
plot(median(squeeze(big_irf(1,:,2,:)),2), 'k--', 'linewidth',3)
hold off

%% Kitagawa-Giacomini for some IRFs
yind = 1;
sind = 1;

r_set = zeros(nlength,1); %center
z_set = zeros(nlength,1); %radius
for hind = 1:1:nlength
    lb = squeeze(big_irf_min(yind,hind,sind,logical(big_irf_valid_minmax)));
    ub = squeeze(big_irf_max(yind,hind,sind,logical(big_irf_valid_minmax)));
    
    alp = 0.70;
    pr_fun = @(r) quantile(max([abs(r-lb),abs(r-ub)], [], 2), alp);
    
    option.Display = 'iter';
    [r,z] = fminbnd(pr_fun,min(lb),max(ub), option);
    
    r_set(hind,:) = r;
    z_set(hind,:) = z;
end
r_set1 = r_set;
z_set1 = z_set;


yind = 1;
sind = 2;

r_set = zeros(nlength,1); %center
z_set = zeros(nlength,1); %radius
for hind = 1:1:nlength
    lb = squeeze(big_irf_min(yind,hind,sind,logical(big_irf_valid_minmax)));
    ub = squeeze(big_irf_max(yind,hind,sind,logical(big_irf_valid_minmax)));
    
    alp = 0.70;
    pr_fun = @(r) quantile(max([abs(r-lb),abs(r-ub)], [], 2), alp);
    
    option.Display = 'iter';
    [r,z] = fminbnd(pr_fun,min(lb),max(ub), option);
    
    r_set(hind,:) = r;
    z_set(hind,:) = z;
end
r_set2 = r_set;
z_set2 = z_set;

figure(1234)
plot([r_set1-z_set1, r_set1+z_set1], 'b')
hold on
plot(r_set1, 'b', 'linewidth', 3);
plot([r_set2-z_set2, r_set2+z_set2], 'r')
plot(r_set2, 'r', 'linewidth', 3);
hold off

