% main_random_initial
% to have a initial point for the random initial point for GIBBS


% main file to save data for two step estimation
% Last revised: 2017-12-11
clc; clear all; close all;
clear -mex;

addpath('HalfVectorization');
addpath('data');
addpath('prior');
addpath('results');
% setenv('MW_MINGW64_LOC','C:\TDM-GCC-64')

is_office = 0; %0=Matlab2016 (laptop), 1=Matlab2017a (office), 2=HPCC
% if is_office == 0
%     addpath('cholinv');
% elseif is_office == 1
%     addpath('cholinv_office');
% end

%% Path control
workpath0 = pwd;
savepath0 = [workpath0, filesep, 'results_2step'];
chk_dir(savepath0);

datapath0 = [workpath0, filesep, 'data'];

%% Option specific to twostep
tsopt.nburn  = 500;
tsopt.nsimul = 10000; %var-gibbs draws

tsopt.nburn2 = 1000;
tsopt.nevery = 9;
tsopt.Qsimul = 1;
tsopt.Gsimul = 1000;

%% Load option
disp('load draws');
loadopt.level = 1; %1: do level, 0: do diff
loadopt.prior = 0; % "0" is prior=11 in the previous code
loadopt.sam   = 1; %sam=1, starts from 1973M1, sam=2 does not include ZLB, sam=3 after great moderation
loadopt.data  = 1;
dataname = ['d', num2str(loadopt.data)];
loadopt.gbrun = 2; %mcmc-seq-index, 1: initial attempt(y(97)), 2:Full sample, 3:shorter sample
loadopt.nlag  = 12; %VAR lag order
loadopt.f     = 1;    %log(diag), 2 diag(chol)

% % load only one batch
% specname = ['pg_lev',num2str(loadopt.level), ...
%     'f',num2str(loadopt.f), 'p',num2str(loadopt.prior), 'sam', num2str(loadopt.sam), ...
%     dataname, 'lag', num2str(loadopt.nlag), 'gbrun',num2str(loadopt.gbrun)];
% load([specname, '.mat'])
% big_SIG = gb_SIG;

% load multiple batches
big_SIG = [];
big_A   = [];
big_B   = [];
big_C   = [];
big_nu  = [];
big_PHI = [];
big_mu  = [];


for loadind = 1:1:10
    loadopt.gbrun = loadind;
    specname = ['pg_lev',num2str(loadopt.level), ...
        'f',num2str(loadopt.f), 'p',num2str(loadopt.prior), 'sam', num2str(loadopt.sam), ...
        dataname, 'lag', num2str(loadopt.nlag), 'gbrun',num2str(loadopt.gbrun)];
    cd(savepath0);
    load([specname, '.mat'])
    cd(workpath0);
    
    big_SIG = cat(4, big_SIG, gb_SIG(:,:,:,tsopt.nburn+1:end));

    big_nu = cat(1, big_nu, gb_nu(tsopt.nburn+1:end,:));
    big_A = cat(3, big_A, gb_A(:,:,tsopt.nburn+1:end));
    big_C = cat(3, big_C, gb_C(:,:,tsopt.nburn+1:end));
    
    big_PHI = cat(3, big_PHI, gb_PHI(:,:,tsopt.nburn+1:end));
    big_mu = cat(1, big_mu, gb_mu(tsopt.nburn+1:end,:));
    big_B = cat(3, big_B, gb_B(:,:,tsopt.nburn+1:end));

end

%% posterior mean and variance
rini = [];

% nu
rini.nu.m = mean(big_nu);
rini.nu.sd = std(big_nu);

% A
big_A = sqz3(shiftdim(big_A,2));
rini.A.m = mean(big_A)';
rini.A.sd = cholcov(cov(big_A))';

% B
big_B = sqz3(shiftdim(big_B,2));
rini.B.m = mean(big_B)';
rini.B.sd = cholcov(cov(big_B))';

% C (only lower triangular part)
big_C = sqz3(shiftdim(big_C,2));
rini.C.m = mean(big_C(:,logical(vec(tril(ones(4)))')))';
rini.C.sd = cholcov(cov(big_C(:,logical(vec(tril(ones(4)))'))))';

% PHI
big_PHI = sqz3(shiftdim(big_PHI,2));
rini.PHI.m = mean(big_PHI)';
rini.PHI.sd = cholcov(cov(big_PHI))';

% mu
rini.mu.m = mean(big_mu)';
rini.mu.sd = cholcov(cov(big_mu))';

%% Save initial moments
save('rini_20171216.mat', 'rini');





