function [p_sm, p_sminv, p_neff] = pgibbs_caiw_fast7e_mex(p_sm0, yt, xt, nu, A, C, mu, PHI, B, V0inv, pg_c2r, pg_snr, pg_uu, pg_uu0) %#codegen

% coder.extrinsic('invChol_mex');

% fast2 uses invChol_mex
% fast3 improves code and add p_neff
% fast3e uses "brute force" proposal
% fast7e uses pdinv instead of invChol_mex

% pg_c2r = zeros(ndim,np,nobs);
% pg_c2r(:,:,1) = chi2rnd( repmat((nu0-(0:ndim-1))', [1,np]));
% pg_c2r(:,:,2:end) = chi2rnd( repmat((nu-(0:ndim-1))', [1,np,nobs-1])); %ndim*np*nobs, for wishart
% pg_snr = randn(ndim*(ndim-1)/2,np,nobs); % for wishart
% pg_uu  = rand(np,nobs); % for resampling
% pg_uu0 = rand(nobs,1);  % for smoothing


% actual particle gibbs
% [nobs, ndim] = size(yt);
[ndim,np,nobs] = size(pg_c2r);

% matrix to store
p_st    = zeros(ndim,ndim,np,nobs);
p_stinv = zeros(ndim,ndim,np,nobs);
p_wnt   = zeros(nobs,np);
p_neff  = zeros(nobs,1);

% ---------
% step 1: Time t=1
t = 1;
y1 = yt(t,:)';
x1 = xt(t,:)';
pT1dqT1 = zeros(np,1);
pM1     = zeros(np,1);
for i=1:np
    % brute-force proposal
    if i>1 %conditional SMC
        [S0, S0sqrt]  = pdinv(V0inv); % S0=S0sqrt*S0sqrt'
        %S0chol = chol(S0);
        %temp_Sigtinv = mywishrnd(S0, S0chol, pg_c2r(:,i,t), pg_snr(:,i,t));
        temp_Sigtinv = mywishrnd(S0, S0sqrt', pg_c2r(:,i,t), pg_snr(:,i,t));
        
        temp_Sigt    = pdinv( temp_Sigtinv );
    else
        temp_Sigt = p_sm0(:,:,t);
        temp_Sigtinv = pdinv(temp_Sigt);
    end
    p_stinv(:,:,i,t) = temp_Sigtinv;
    p_st(:,:,i,t) = temp_Sigt;
    
    % for brute-force proposal
    pT1dqT1(i) = 0;
    
    % evaluation of measurement density
    % fun_mean = log(diag())
    pM1(i) = logmvnpdf(y1, mu + PHI*x1 + B*log(diag(temp_Sigt)), temp_Sigt, temp_Sigtinv);
end
logw1 = pM1+pT1dqT1; %log importance weight
w1 = exp(logw1); %importance weight
wn1 = w1/sum(w1); %normalized importance weight

p_wnt(t,:) = wn1;
p_neff(t,:) = 1/sum(wn1.^2);

% step 2: Recursion
for t=2:nobs
    y1 = yt(t,:)';
    x1 = xt(t,:)';
    pM1 = zeros(np,1);
    pT1dqT1 = zeros(np,1);
    
    % a) resampling
    uu = pg_uu(:,t);
    [indx] = multinomial_resampling( wn1, uu );
    indx(1) = 1; % conditional SMC, first particle is always a previous draw
    Sig0 = p_st(:,:,indx,t-1);
    
    % b) propose
    for i=1:np
        
        % complicated proposal
        Sigtbar = C + A*(Sig0(:,:,i))*A'; %conditional mean
        S0inv = (nu-ndim-1)*Sigtbar;
        
        if i>1 % conditional SMC
            % brute-force proposal
            %S0  = pdinv(S0inv);
            %S0chol = chol(S0);
            %temp_Sigtinv = mywishrnd(S0, S0chol, pg_c2r(:,i,t), pg_snr(:,i,t));
            
            [S0, S0sqrt]  = pdinv(S0inv); % S0=S0sqrt*S0sqrt'
            temp_Sigtinv = mywishrnd(S0, S0sqrt', pg_c2r(:,i,t), pg_snr(:,i,t));
            
            temp_Sigt    = pdinv( temp_Sigtinv );
        else
            temp_Sigt    = p_sm0(:,:,t);
            temp_Sigtinv = pdinv(temp_Sigt);
        end
        p_stinv(:,:,i,t) = temp_Sigtinv;
        p_st(:,:,i,t) = temp_Sigt;
        
        % for brute-force proposal
        pT1dqT1(i) = 0;
        % evaluation of measurement density
        % fun_mean = log(diag())
        pM1(i) = logmvnpdf(y1, mu + PHI*x1 + B*log(diag(temp_Sigt)), temp_Sigt, temp_Sigtinv);
    end
    logw1 = pM1 + pT1dqT1; %log importance weight
    w1 = exp(logw1); %importance weight
    wn1 = w1/sum(w1); %normalized importance weight
    
    p_wnt(t,:) = wn1;
    p_neff(t,:) = 1/sum(wn1.^2);
end

% ----------------
% smoothing
p_sm = zeros(ndim,ndim,nobs);
p_sminv = zeros(ndim,ndim,nobs);

% last period
t  = nobs;
% bt = mnrnd(1,p_wnt(t,:));
bt = multinomial_resampling(p_wnt(t,:), pg_uu0(t));
x1 = p_st(:,:,bt,t);
x1inv = p_stinv(:,:,bt,t);
p_sm(:,:,t) = x1;
p_sminv(:,:,t) = x1inv;

% recursion (backward)
for tt=1:(nobs-1)
    t = nobs-tt;
    
    pT1 = zeros(1,np);
    for i=1:np
        Sigtbar = C + A*(p_st(:,:,i,t))*A';
        S0inv = (nu-ndim-1)*Sigtbar;
        pT1(i) = logiwishpdf_propto(x1, S0inv, nu, x1inv);
    end
    w0 = p_wnt(t,:).*exp(pT1);
    wn0 = w0/sum(w0);
    %bt = mnrnd(1,wn0);
    bt = multinomial_resampling(wn0, pg_uu0(t));
    
    x1 = p_st(:,:,bt,t);
    x1inv = p_stinv(:,:,bt,t);
    p_sm(:,:,t) = x1;
    p_sminv(:,:,t) = x1inv;
    
end


