function [prior, mhtune] = setup_prior0(estopt)
% set-up prior and mhtunes

ndim = estopt.ndim;
ldim = estopt.ldim;
nlag = estopt.nlag;
nobs = estopt.nobs;
nshock = ndim*(ndim+1)/2;


%% Others
if estopt.f == 1
    prior.fun_mean = @(sigmat) log(diag(sigmat)); % Note that we fix B to zero inside of CAW function. 
elseif estopt.f ==2
    prior.fun_mean = @(sigmat) diag(chol(sigmat,'lower')); % f(Sigma(t)) in the paper
end

%% Prior
% prior - SIG0
L = full(EliminationM(ndim));
D = full(DuplicationM(ndim));
prior.SIG0.v  = 10;
prior.SIG0.L  = L;
prior.SIG0.D  = D;

% prior - nu
prior.nu.m = 40;
prior.nu.V = 10;
prior.nu.a = prior.nu.m^2/prior.nu.V;
prior.nu.b = prior.nu.V/prior.nu.m;
prior.nu.fun = @(para)gampdf(para,prior.nu.a, prior.nu.b);
% ezplot(prior.nu.fun, [0,100])
% aaa = gamrnd(prior.nu.a, prior.nu.b, [1000,1]);
% mean(aaa)
% var(aaa)

% nu - griddy gibbs setting
prior.nu.grid0 = (ndim+2);
prior.nu.grid1 = 70;
prior.nu.ngrid = 500;
prior.nu.grid = linspace(prior.nu.grid0, prior.nu.grid1, prior.nu.ngrid)';

% Prior for A - independent across elements
% prior.A.m  = diag([0.9, 0.9]);
prior.A.m  = 0.9*eye(ndim);
if estopt.pnum == 1 || estopt.pnum == 2
    prior.A.sd = 0.1*ones(ndim);
else
    prior.A.sd = 0.25*ones(ndim);
end

% prior - C
% C = diag([0.9, 0.15, 1.0, 0.3])/20;
% prior.C.df   = 1+ndim+10;
% prior.C.C    = (prior.C.df-ndim-1)*C;
% prior.C.Cinv = inv(prior.C.C);
% C = eye(ndim);

% data dependent way but with large variance
Atilde = L*kron(prior.A.m,prior.A.m)*D;
if estopt.pnum == 1 || estopt.pnum == 2
    temp_cov = diag(diag(cov(estopt.VARerrors)));
else
    temp_cov = cov(estopt.VARerrors);
end
C = vech2mat( (eye(nshock)-Atilde)*vech(temp_cov) );

if estopt.pnum == 1 || estopt.pnum == 2
    prior.C.df   = ndim+11;
else
    prior.C.df   = ndim+2;
end
prior.C.C    = (prior.C.df-ndim-1)*C;
prior.C.Cinv = inv(prior.C.C);

Atilde = L*kron(prior.A.m,prior.A.m)*D;
Ctilde = L*C(1:end)';
longrunmean = (vech2mat((eye(nshock)-Atilde)\Ctilde)); %Unconditional variance of innovation
longrunmean
% var(errors) 0.432437755354937        0.0450296933987317         0.210171535891768        0.0556412366944422

%% Prior for mu, PHI, B
% estopt.prior = 11
% prior mu
prior.m = zeros(ndim,1); % mean(mu)
prior.Minv = inv(1000*eye(ndim));
prior.M    = inv(prior.Minv);

% prior B
prior.b = zeros(ndim,ndim);
prior.b = prior.b(:);

prior.B = 10*ones(ndim,ldim);
if estopt.pnum == 2
    prior.B(ndim,1:ldim) = 0.1;
else
    prior.B(ndim,1:ldim) = 0.05;
end

prior.B = diag(prior.B(:));

% prior PHI - Minnestoa-type priors
prior.phi1 = eye(ndim);
prior.phi = [prior.phi1(:); zeros(ndim*ndim*(nlag-1),1)]; %mean([PHI(1,:)'; PHI(2,:)' ...]
% Setting up true Minnesota prior (Sims and Zha 96)
priorphistd = zeros(ndim*ndim*nlag,1);
stdshocks = repmat(estopt.stdshocks,ndim,1);
stdshocks2temp = repmat(estopt.stdshocks',ndim,1);
stdshocks2 = stdshocks2temp(:); % creating a long vector of [std1;std1;std1;std1;std2;std2;std2;,...]
for ii = 1:nlag
    priorphistd((ii-1)*ndim*ndim+1:ii*(ndim*ndim)) = 0.2/ii*stdshocks./stdshocks2;
end
prior.PHI = diag(priorphistd.^2);
% prior.PHI = 100*eye(length(prior.phi));

% collecting mu,B,PHI priors
prior.muPHIB.m = [prior.m; prior.phi; prior.b]; %prior
prior.muPHIB.V = blkdiag(prior.M, prior.PHI, prior.B); %prior

% ------
% Prior if data are not in level
if estopt.level == 0
    prior.phi1 = zeros(ndim);
    prior.phi  = [prior.phi1(:);zeros(ndim*ndim*(nlag-1),1)]; %mean([PHI(1,:)'; PHI(2,:)'])
    prior.PHI  = 100*eye(ndim*ndim*nlag); %var([PHI(1,:)'; PHI(2,:)'])

    % collecting mu,B,PHI priors
    prior.muPHIB.m = [prior.m; prior.phi; prior.b]; %prior
    prior.muPHIB.V = blkdiag(prior.M, prior.PHI, prior.B); %prior
end

%% mh-tuning
% prop.sd_S  = 0.04;
% mhtune.S.p = 0.0; %mixture prob in mixture proposal
% mhtune.S.iwscale = ones(nobs,1)*100; %higher -> low variance, used to be 100
% mhtune.SIG0.iwscale  = 100; %higher -> low variance, used to be 100
% 
% prop_sd_nu = 0.2;
% mhtune.nu.a = 0.30;  %target acceptance rate
% mhtune.nu.c = 0.55;  %cooling rate (between 0.5 and 1), c = inf means no adaptation
% mhtune.nu.b = 10000; %bound
% mhtune.nu.logvar = log(prop_sd_nu^2); %log proposal variance (initial proposal variance)

prop_sd_C  = 0.02;
mhtune.C.a = 0.30;  %target acceptance rate
mhtune.C.c = 0.55;  %cooling rate (between 0.5 and 1), c = inf means no adaptation
mhtune.C.b = 10000; %bound
mhtune.C.logvar = log(prop_sd_C^2)*ones(ndim*(ndim+1)/2,1); %log proposal variance (initial proposal variance)

prop_sd_A  = 0.02;
mhtune.A.a = 0.30;  %target acceptance rate
mhtune.A.c = 0.55;  %cooling rate (between 0.5 and 1), c = inf means no adaptation
mhtune.A.b = 10000; %bound
mhtune.A.logvar = log(prop_sd_A^2)*ones(ndim^2,1); %log proposal variance (initial proposal variance)
