function [prior, mhtune] = setup_prior_minn5(estopt)
% set-up prior and mhtunes

ndim = estopt.ndim;
ldim = estopt.ldim;
nlag = estopt.nlag;
nobs = estopt.nobs;
nshock = ndim*(ndim+1)/2;


%% Others
if estopt.f == 1
    prior.fun_mean = @(sigmat) log(diag(sigmat)); % Note that we fix B to zero inside of CAW function. 
elseif estopt.f ==2
    prior.fun_mean = @(sigmat) diag(chol(sigmat,'lower')); % f(Sigma(t)) in the paper
end

%% Prior
% prior - SIG0
L = full(EliminationM(ndim));
D = full(DuplicationM(ndim));
prior.SIG0.v  = 10;
prior.SIG0.L  = L;
prior.SIG0.D  = D;

% prior - nu
if estopt.prior == 0
    prior.fixnu = 1;
    prior.nuval = 40;
    prior.nu.m = nan;
    prior.nu.V = nan;
elseif estopt.prior == 1
    prior.nu.m = 40;
    prior.nu.V = 40;
    prior.fixnu = 0;
elseif estopt.prior == 2
    prior.nu.m = 40;
    prior.nu.V = 5;
    prior.fixnu = 0;
elseif estopt.prior == 3
    prior.nu.m = 40;
    prior.nu.V = 1;
    prior.fixnu = 0;
elseif estopt.prior == 4
    prior.nu.m = 40;
    prior.nu.V = 1;
    prior.fixnu = 0;
elseif estopt.prior == 5
    prior.nu.m = 40;
    prior.nu.V = 1;
    prior.fixnu = 0;    
elseif estopt.prior == 6
    prior.nu.m = 40;
    prior.nu.V = 1;
    prior.fixnu = 0;   
elseif estopt.prior == 7
    prior.nu.m = 40;
    prior.nu.V = 1;
    prior.fixnu = 0;    
elseif estopt.prior == 8 || estopt.prior == 10 || estopt.prior == 11 || estopt.prior == 12 || estopt.prior == 13
    prior.nu.m = 40;
    prior.nu.V = 1;
    prior.fixnu = 0;    
elseif estopt.prior == 9
    prior.nu.m = 40;
    prior.nu.V = 1;
    prior.fixnu = 0;
elseif estopt.prior == 14 %We set mean of nu to be lower...?
    prior.nu.m = 20;
    prior.nu.V = 1;
    prior.fixnu = 0;
end
prior.nu.a = prior.nu.m^2/prior.nu.V;
prior.nu.b = prior.nu.V/prior.nu.m;
prior.nu.fun = @(para)gampdf(para,prior.nu.a, prior.nu.b);
% ezplot(prior.nu.fun, [0,100])
% aaa = gamrnd(prior.nu.a, prior.nu.b, [1000,1]);
% mean(aaa)
% var(aaa)
% nu - griddy gibbs setting
prior.nu.grid0 = (ndim+2);
prior.nu.grid1 = 100;
prior.nu.ngrid = 1000;
prior.nu.grid = linspace(prior.nu.grid0, prior.nu.grid1, prior.nu.ngrid)';

% prior - C
% C = diag([0.9, 0.15, 1.0, 0.3])/20;
% prior.C.df   = 1+ndim+10;
% prior.C.C    = (prior.C.df-ndim-1)*C;
% prior.C.Cinv = inv(prior.C.C);

% C = eye(ndim);
% prior.C.df   = ndim+2;
% prior.C.C    = C;
% prior.C.Cinv = inv(prior.C.C);

if estopt.prior == 12
C = diag((0.3*ones(length(estopt.stdshocks),1)).^2)*0.19; %0.19 = 1-0.9^2: this is to set the unconditional variance of the innov to be equal to the var of the errors from OLS
else
C = diag(estopt.stdshocks.^2)*0.19; %0.19 = 1-0.9^2: this is to set the unconditional variance of the innov to be equal to the var of the errors from OLS
end

if estopt.prior == 12
    prior.C.df   = ndim+1+10;
elseif estopt.prior == 13
    prior.C.df   = ndim+4; %lowest degree of freedom value that still allows for a variance
else
    prior.C.df   = 1+ndim+10;
end
prior.C.C    = (prior.C.df-ndim-1)*C;
prior.C.Cinv = inv(prior.C.C);


% Prior for A - independent across elements
prior.A.m  = diag(0.9*ones(ndim,1));
prior.A.sd = 0.1*ones(ndim);

Atilde = L*kron(prior.A.m,prior.A.m)*D;
Ctilde = L*C(1:end)';
longrunmean = (vech2mat((eye(nshock)-Atilde)\Ctilde)); %Unconditional variance of innovation
longrunmean
% var(errors) 0.432437755354937        0.0450296933987317         0.210171535891768        0.0556412366944422

% Prior for mu, PHI, B
% prior mu
prior.m    = zeros(ndim,1);       %mean(mu)
% prior.Minv = inv(0.2*diag(estopt.stdshocks.^2));  %var(mu)
if estopt.prior == 8 || estopt.prior == 10 || estopt.prior == 11 || estopt.prior == 12  || estopt.prior == 13 || estopt.prior == 14
prior.Minv = inv(1000*eye(ndim));
prior.M    = inv(prior.Minv);
elseif estopt.prior == 9
prior.Minv = inv(0.2^2*eye(ndim));
prior.M    = inv(prior.Minv);
end

prior.b    = zeros(ndim,ldim);
prior.b    = prior.b(:);

if estopt.prior == 10 %Loose B prior
    prior.B    = 10*ones(ndim,ldim);
    prior.B(ndim,1:ldim) = 0.5;
    prior.B = diag(prior.B(:));
elseif estopt.prior == 11 || estopt.prior == 12 || estopt.prior == 13 || estopt.prior == 14
    prior.B    = 10*ones(ndim,ldim);
    prior.B(ndim,1:ldim) = 0.05;
    prior.B = diag(prior.B(:));
else
    prior.B    = 1*eye(ndim*ldim);
end

% Prior for PHI, Set Minnesota-type priors if IP and price level are in levels!
if estopt.level == 0
    prior.phi1 = zeros(ndim);
elseif estopt.level == 1
    prior.phi1 = eye(ndim);
end
% prior.phi1 = 0.8*eye(ndim);
prior.phi  = [prior.phi1(:);zeros(ndim*ndim*(nlag-1),1)]; %mean([PHI(1,:)'; PHI(2,:)'])
% prior.PHI  = 100*eye(ndim*ndim*nlag); %var([PHI(1,:)'; PHI(2,:)'])
% Setting up true Minnesota prior (Sims and Zha 96)
priorphistd = zeros(ndim*ndim*nlag,1);
stdshocks = repmat(estopt.stdshocks,ndim,1);
stdshocks2temp = repmat(estopt.stdshocks',ndim,1);
stdshocks2 = stdshocks2temp(:); % creating a long vector of [std1;std1;std1;std1;std2;std2;std2;,...]
for ii = 1:nlag
    priorphistd((ii-1)*ndim*ndim+1:ii*(ndim*ndim)) = 0.2/ii*stdshocks./stdshocks2;
end
prior.PHI = diag(priorphistd.^2);
% prior.PHI = 100*eye(length(prior.phi));

% collecting mu,B,PHI priors
prior.muPHIB.m = [prior.m; prior.phi; prior.b]; %prior
prior.muPHIB.V = blkdiag(prior.M, prior.PHI, prior.B); %prior


%% mh-tuning
% prop.sd_S  = 0.04;
% mhtune.S.p = 0.0; %mixture prob in mixture proposal
% if estopt.data == 5
%     mhtune.S.iwscale = ones(nobs,1)*200; %higher -> low variance, used to be 100
%     mhtune.SIG0.iwscale  = 200; %higher -> low variance, used to be 100
% else
%     mhtune.S.iwscale = ones(nobs,1)*100; %higher -> low variance, used to be 100
%     mhtune.SIG0.iwscale  = 100; %higher -> low variance, used to be 100
% end
% 
% prop_sd_nu = 0.2;
% mhtune.nu.a = 0.30;  %target acceptance rate
% mhtune.nu.c = 0.55;  %cooling rate (between 0.5 and 1), c = inf means no adaptation
% mhtune.nu.b = 10000; %bound
% mhtune.nu.logvar = log(prop_sd_nu^2); %log proposal variance (initial proposal variance)

prop_sd_C  = 0.02;
mhtune.C.a = 0.30;  %target acceptance rate
mhtune.C.c = 0.55;  %cooling rate (between 0.5 and 1), c = inf means no adaptation
mhtune.C.b = 10000; %bound
mhtune.C.logvar = log(prop_sd_C^2)*ones(ndim*(ndim+1)/2,1); %log proposal variance (initial proposal variance)

prop_sd_A  = 0.02;
mhtune.A.a = 0.30;  %target acceptance rate
mhtune.A.c = 0.55;  %cooling rate (between 0.5 and 1), c = inf means no adaptation
mhtune.A.b = 10000; %bound
mhtune.A.logvar = log(prop_sd_A^2)*ones(ndim^2,1); %log proposal variance (initial proposal variance)
