function [prior, mhtune] = setup_prior_train(estopt)
% set-up prior and mhtunes

ndim = estopt.ndim;
ldim = estopt.ldim;
nlag = estopt.nlag;
nobs = estopt.nobs;
nshock = ndim*(ndim+1)/2;


%% Others
if estopt.f == 1
    prior.fun_mean = @(sigmat) log(diag(sigmat)); % Note that we fix B to zero inside of CAW function. 
elseif estopt.f ==2
    prior.fun_mean = @(sigmat) diag(chol(sigmat,'lower')); % f(Sigma(t)) in the paper
end

%% Prior
% prior - SIG0
L = full(EliminationM(ndim));
D = full(DuplicationM(ndim));
prior.SIG0.v  = 10;
prior.SIG0.L  = L;
prior.SIG0.D  = D;

% prior - nu
prior.nu.m = 40;
prior.nu.V = 1;
% if estopt.prior == 1
%     prior.nu.m = 40;
%     prior.nu.V = 40;
% elseif estopt.prior == 2
%     prior.nu.m = 40;
%     prior.nu.V = 5;
% elseif estopt.prior == 3
%     prior.nu.m = 40;
%     prior.nu.V = 1;
% end
prior.nu.a = prior.nu.m^2/prior.nu.V;
prior.nu.b = prior.nu.V/prior.nu.m;
prior.nu.fun = @(para)gampdf(para,prior.nu.a, prior.nu.b);
% ezplot(prior.nu.fun, [0,100])
% aaa = gamrnd(prior.nu.a, prior.nu.b, [1000,1]);
% mean(aaa)
% var(aaa)

% prior - C
C = diag([0.9, 0.15, 1.0, 0.3])/20;
prior.C.df   = 1+ndim+1;
prior.C.C    = (prior.C.df-ndim-1)*C;
prior.C.Cinv = inv(prior.C.C);

% Prior for A - independent across elements
prior.A.m  = diag([0.9, 0.9, 0.9, 0.9]);
prior.A.sd = 0.1*ones(ndim);

Atilde = L*kron(prior.A.m,prior.A.m)*D;
Ctilde = L*C(1:end)';
longrunmean = (vech2mat((eye(nshock)-Atilde)\Ctilde)); %Unconditional variance of innovation
longrunmean
% var(errors) 0.432437755354937        0.0450296933987317         0.210171535891768        0.0556412366944422

% Prior for mu, PHI, B
% prior mu
prior.m    = zeros(ndim,1);       %mean(mu)
prior.Minv = inv(1*eye(ndim));  %var(mu)
prior.M    = inv(prior.Minv);

% prior for B
prior.b    = zeros(ndim*ldim,1);  %mean([B(1,:)'; B(2,:)'])
prior.B    = 1*eye(ndim*ldim); %var([B(1,:)'; B(2,:)'])

% Prior for PHI, Set Minnesota-type priors if IP and price level are in levels!
if estopt.level == 0
    prior.phi1 = zeros(ndim);
elseif estopt.level == 1
    prior.phi1 = eye(ndim);
end
% prior.phi1 = 0.8*eye(ndim);
prior.phi  = [prior.phi1(:);zeros(ndim*ndim*(nlag-1),1)]; %mean([PHI(1,:)'; PHI(2,:)'])
prior.PHI  = 0.1*eye(ndim*ndim*nlag); %var([PHI(1,:)'; PHI(2,:)'])

% collecting mu,B,PHI priors
prior.muPHIB.m = [prior.m; prior.phi; prior.b]; %prior
prior.muPHIB.V = blkdiag(prior.M, prior.PHI, prior.B); %prior


%% mh-tuning
prop.sd_S  = 0.04;
mhtune.S.p = 0.0; %mixture prob in mixture proposal
mhtune.S.iwscale = ones(nobs,1)*100; %higher -> low variance, used to be 100
mhtune.SIG0.iwscale  = 100; %higher -> low variance, used to be 100

prop_sd_nu = 0.2;
mhtune.nu.a = 0.30;  %target acceptance rate
mhtune.nu.c = 0.55;  %cooling rate (between 0.5 and 1), c = inf means no adaptation
mhtune.nu.b = 10000; %bound
mhtune.nu.logvar = log(prop_sd_nu^2); %log proposal variance (initial proposal variance)

prop_sd_C  = 0.02;
mhtune.C.a = 0.30;  %target acceptance rate
mhtune.C.c = 0.55;  %cooling rate (between 0.5 and 1), c = inf means no adaptation
mhtune.C.b = 10000; %bound
mhtune.C.logvar = log(prop_sd_C^2)*ones(ndim*(ndim+1)/2,1); %log proposal variance (initial proposal variance)

prop_sd_A  = 0.02;
mhtune.A.a = 0.30;  %target acceptance rate
mhtune.A.c = 0.55;  %cooling rate (between 0.5 and 1), c = inf means no adaptation
mhtune.A.b = 10000; %bound
mhtune.A.logvar = log(prop_sd_A^2)*ones(ndim^2,1); %log proposal variance (initial proposal variance)
