% script for estimation of VAR-CAIW using pgibbs

% using mex file

% mexf: fast version (risky)

% mexfr: random initial starting points

%% recover some settings

np = estopt.np;

nsimul  = estopt.nsimul;
nadapt  = estopt.nadapt;

nreport  = estopt.nreport;
nsave1   = estopt.nsave1;
nsave2   = estopt.nsave2;

nsimul1 = nsimul/nsave1;
nsimul2 = nsimul/nsave2;

counter1 = 0;
counter2 = 0;

ndim = estopt.ndim;
ldim = estopt.ldim;
nlag = estopt.nlag;
nobs = estopt.nobs;
nshock = ndim*(ndim+1)/2;

fun_mean = prior.fun_mean; %volatility function

L = prior.SIG0.L;
D = prior.SIG0.D;

% data
yt = YYY;
xt = XXX;


%% Random initial points
disp('Initializing Gibbs ...');
load('rini_20171216.mat');

ini_scale = 2;

% ------------
% nu 
valid = 0;
while ~valid
    nu_old = rini.nu.m + ini_scale*rini.nu.sd*randn;
    if nu_old >0
        valid = 1;
    end
end

% ------------
% C 
valid = 0;
while ~valid
    C_old0 = rini.C.m + ini_scale*rini.C.sd * randn(size(rini.C.sd,2),1);
    C_old = zeros(ndim);
    C_old(tril(ones(ndim))==1) = C_old0;
    for i=1:1:ndim
        for j=i+1:1:ndim
            C_old(i,j) = C_old(j,i);
        end
    end
    
    if min(eig(C_old))>0
        valid = 1;
    end
end

% ------------
% A
valid = 0;
while ~valid
    A_old = rini.A.m + ini_scale*rini.A.sd * randn(ndim*ndim,1);
    A_old = reshape(A_old, ndim, ndim);
    % check identification condition
    if A_old(1,1) >0;
        % check stationarity
        Ctilde_old  = L*C_old(1:end)';
        Atilde_old  = L*kron(A_old, A_old)*D;
        V0inv_old   = (prior.SIG0.v-ndim-1)*(vech2mat((eye(nshock)-Atilde_old)\Ctilde_old));
        if min(eig(V0inv_old))>0;
            valid = 1;
        end
    end
end

% ------------
% B 
B_old = rini.B.m + ini_scale*rini.B.sd * randn(ndim*ndim,1);
B_old = reshape(B_old, ndim, ndim);

% ------------
% mu 
mu_old = rini.mu.m + ini_scale*rini.mu.sd * randn(ndim,1);

% ------------
% PHI (ndim, ndim*nlag)
PHI_old = rini.PHI.m + ini_scale*rini.PHI.sd * randn(size(rini.PHI.m,1),1);
PHI_old = reshape(PHI_old, ndim,ndim*nlag);

%------
% Drawing SIG and SIGinv
SIG_old = zeros(ndim,ndim,nobs);
SIGinv_old = zeros(ndim,ndim,nobs);
for t=1:1:nobs
    SIG_old(:,:,t) = cov(estopt.VARerrors);
    SIGinv_old(:,:,t) = inv(SIG_old(:,:,t));
end

for i0 = 1:1:10
    Ctilde_old  = L*C_old(1:end)';
    Atilde_old  = L*kron(A_old, A_old)*D;
    V0inv_old   = (prior.SIG0.v-ndim-1)*(vech2mat((eye(nshock)-Atilde_old)\Ctilde_old));
    V0_old      = inv(V0inv_old);
    nu0_old = prior.SIG0.v;
    %[SIG_old,SIGinv_old] = pgibbs_caiw_fast2(SIG_old, yt, xt, nu_old, A_old, C_old, mu_old, PHI_old, B_old, prior.SIG0.v, V0_old, V0inv_old, fun_mean, np);
    
    % generate all random variables used in the gibbs (for mex)
    pg_c2r = zeros(ndim,np,nobs);
    pg_c2r(:,:,1) = chi2rnd( repmat((nu0_old-(0:ndim-1))', [1,np])); %for wishart
    pg_c2r(:,:,2:end) = chi2rnd( repmat((nu_old-(0:ndim-1))', [1,np,nobs-1])); %ndim*np*nobs, for wishart
    pg_snr = randn(ndim*(ndim-1)/2,np,nobs); % for wishart
    pg_uu  = rand(np,nobs); % for resampling
    pg_uu0 = rand(nobs,1);  % for smoothing
    
    % [SIG_old, SIGinv_old, p_neff] = pgibbs_caiw_fast7e_mex(SIG_old, yt, xt, nu_old, A_old, C_old, mu_old, PHI_old, B_old, V0inv_old, pg_c2r, pg_snr, pg_uu, pg_uu0);
    % a = shiftdim(SIG_old,2);
    
    % actual filter/smoother
    if is_office == -1
        [SIG_old, SIGinv_old, p_neff] = pgibbs_caiw_fast7e_mex(SIG_old, yt, xt, nu_old, A_old, C_old, mu_old, PHI_old, B_old, V0inv_old, pg_c2r, pg_snr, pg_uu, pg_uu0);
    elseif is_office == 0
        [SIG_old, SIGinv_old, p_neff] = pgibbs_caiw_fast7e_mex2016(SIG_old, yt, xt, nu_old, A_old, C_old, mu_old, PHI_old, B_old, V0inv_old, pg_c2r, pg_snr, pg_uu, pg_uu0);
    elseif is_office == 1
        [SIG_old, SIGinv_old, p_neff] = pgibbs_caiw_fast7e_mex2017(SIG_old, yt, xt, nu_old, A_old, C_old, mu_old, PHI_old, B_old, V0inv_old, pg_c2r, pg_snr, pg_uu, pg_uu0);
    elseif is_office == 2
        [SIG_old, SIGinv_old, p_neff] = pgibbs_caiw_fast7e_mex_hpcc(SIG_old, yt, xt, nu_old, A_old, C_old, mu_old, PHI_old, B_old, V0inv_old, pg_c2r, pg_snr, pg_uu, pg_uu0);
    end
end


%% Gibbs simulation
disp('Actual Gibbs ...');

temp_rejA = 0;
temp_rejC = 0;

gb_nu  = zeros(nsimul1,1);
gb_SIG = zeros(ndim,ndim,nobs,nsimul2);

gb_A   = zeros(ndim,ndim,nsimul1);
gb_C   = zeros(ndim,ndim,nsimul1);

gb_rejA = zeros(nsimul1,ndim^2);
gb_rejC = zeros(nsimul1,nshock);

gb_mu  = zeros(nsimul1,ndim);
gb_B   = zeros(ndim,ldim,nsimul1);
gb_PHI = zeros(ndim,ndim*nlag,nsimul1);

gb_neff    = zeros(nsimul1,3); %min, median, max
gb_neffind = zeros(nsimul1,2); %min, mmax

gb_time = tic;
for simulind = 1:1:nsimul
    
    %------
    % drawing state
    Ctilde_old  = L*C_old(1:end)';
    Atilde_old  = L*kron(A_old, A_old)*D;
    V0inv_old   = (prior.SIG0.v-ndim-1)*(vech2mat((eye(nshock)-Atilde_old)\Ctilde_old));
    V0_old      = inv(V0inv_old);
    nu0_old = prior.SIG0.v;
    %[SIG_old,SIGinv_old] = pgibbs_caiw_fast2(SIG_old, yt, xt, nu_old, A_old, C_old, mu_old, PHI_old, B_old, prior.SIG0.v, V0_old, V0inv_old, fun_mean, np);
    
    % generate all random variables used in the gibbs (for mex)
    pg_c2r = zeros(ndim,np,nobs);
    pg_c2r(:,:,1) = chi2rnd( repmat((nu0_old-(0:ndim-1))', [1,np])); %for wishart
    pg_c2r(:,:,2:end) = chi2rnd( repmat((nu_old-(0:ndim-1))', [1,np,nobs-1])); %ndim*np*nobs, for wishart
    pg_snr = randn(ndim*(ndim-1)/2,np,nobs); % for wishart
    pg_uu  = rand(np,nobs); % for resampling
    pg_uu0 = rand(nobs,1);  % for smoothing
   
    % actual filter/smoother
    if is_office == -1
        [SIG_old, SIGinv_old, p_neff] = pgibbs_caiw_fast7e_mex(SIG_old, yt, xt, nu_old, A_old, C_old, mu_old, PHI_old, B_old, V0inv_old, pg_c2r, pg_snr, pg_uu, pg_uu0);
    elseif is_office == 0
        [SIG_old, SIGinv_old, p_neff] = pgibbs_caiw_fast7e_mex2016(SIG_old, yt, xt, nu_old, A_old, C_old, mu_old, PHI_old, B_old, V0inv_old, pg_c2r, pg_snr, pg_uu, pg_uu0);
    elseif is_office == 1
        [SIG_old, SIGinv_old, p_neff] = pgibbs_caiw_fast7e_mex2017(SIG_old, yt, xt, nu_old, A_old, C_old, mu_old, PHI_old, B_old, V0inv_old, pg_c2r, pg_snr, pg_uu, pg_uu0);
    elseif is_office == 2
        [SIG_old, SIGinv_old, p_neff] = pgibbs_caiw_fast7e_mex_hpcc(SIG_old, yt, xt, nu_old, A_old, C_old, mu_old, PHI_old, B_old, V0inv_old, pg_c2r, pg_snr, pg_uu, pg_uu0);
    end
    %------
    % drawing nu
    nu_old = draw_nu_griddy(SIG_old, SIGinv_old, A_old, C_old, prior);
    
    %------
    % drawing A
    [A_old, oinfo, mhtune] = draw_A_new(SIG_old, SIGinv_old, nu_old, A_old, C_old, prior, mhtune, L, D);
    % -------------
    % Adapting A
    if simulind < nadapt
        for ii=1:1:ndim^2
            mhtune.A.logvar(ii) = adaptiveMCMCvar(mhtune.A.logvar(ii), ...
                oinfo.mhratio(ii), mhtune.A.a, mhtune.A.b, mhtune.A.c, simulind);
        end
    end
    temp_rejA = oinfo.rejA;
    
    %------
    % drawing C
    [C_old, oinfo, mhtune] = draw_C_new(SIG_old, SIGinv_old, nu_old, A_old, C_old, prior, mhtune, L, D);
    % -------
    % Adapting C
    if simulind < nadapt
        for ii=1:1:nshock
            mhtune.C.logvar(ii) = adaptiveMCMCvar(mhtune.C.logvar(ii), ...
                oinfo.mhratio(ii), mhtune.C.a, mhtune.C.b, mhtune.C.c, simulind);
        end
    end
    temp_rejC = oinfo.rejC;
    
    % -------
    % drawing mu, PHI, B
    [mu_old, PHI_old, B_old, mhtune] = ...
        draw_muPHIB(yt, xt, SIG_old, ...
        ndim, nlag, ldim, prior, mhtune, nobs, fun_mean);
    
    % -----
    % store
    if rem(simulind, nsave1) == 0
        counter1 = counter1 + 1;
        
        gb_nu(counter1,1) = nu_old;
        
        gb_A(:,:,counter1) = A_old;
        gb_rejA(counter1,:) = temp_rejA;
        
        gb_C(:,:,counter1) = C_old;
        gb_rejC(counter1,:) = temp_rejC;
        
        gb_mu(counter1,:)    = mu_old;
        gb_B(:,:,counter1)   = B_old;
        gb_PHI(:,:,counter1) = PHI_old;
        
        [min_neff,min_neff_ind] = min(p_neff);
        [max_neff,max_neff_ind] = max(p_neff);
        gb_neff(counter1,:) = [min_neff, mean(p_neff), max_neff]; %min, mean, max
        gb_neffind(counter1,:) = [min_neff_ind,max_neff_ind];
    end
    
    if rem(simulind, nsave2) == 0
        counter2 = counter2 + 1;
        gb_SIG(:,:,:,counter2) = SIG_old;
    end
    
    % -----
    % report
    if rem(simulind,nreport) == 0
        disp('------------------------------------------------');
        disp(['Draws  : ', num2str(simulind), ' / ', num2str(nsimul)]);
        disp(['Time left (sec)     : ', num2str((toc(gb_time)/simulind)*(nsimul-simulind))]);
        disp(['Time per draw (sec) : ', num2str((toc(gb_time)/simulind))]);
        disp('------------------------------------------------');
        disp(['rej neff  :', num2str(mean(gb_neff(1:counter1,:)))]);
        disp(['rej A  :', num2str(mean(gb_rejA(1:counter1,:)))]);
        disp(['rej C  :', num2str(mean(gb_rejC(1:counter1,:)))]);
        disp('------------------------------------------------');
        disp(['mean of nu : '])
        disp(mean(gb_nu(1:counter1,1)))
        disp(['mean of A : '])
        disp(mean(gb_A(:,:,1:counter1),3))
        disp(['mean of C : '])
        disp(mean(gb_C(:,:,1:counter1),3))
        disp('------------------------------------------------');
    end
end
