%JBES version: This code produces the IRFs for the Carriero, Clark, Marcellino (2017) model (Example 2 in the text)
priordata = 0;

addpath('drawsofparamsandstates');
addpath('toolbox_plot');

if priordata == 0
    loadintomatlab;
end

%% This code calculates the Cholesky case
nlength = 40;
nlag    = 6;
nvollag = 2;
nirf    = 1;
nirf2   = 2;
Q = eye(2); %Cholesky case

macroirf = zeros(30,nlength,ndraws);
finirf   = zeros(30,nlength,ndraws);

macrosirf = zeros(2,nlength,ndraws);
finsirf   = zeros(2,nlength,ndraws);

for i = 1:ndraws
    i
    Ppi        = Pimat(:,:,i);
    Ppi        = Ppi';
    factorcoef = factorcoefmat(:,:,i);
    factorcoef = factorcoef';
    Phif       = PhiFmat(:,:,i);
    
    [macroirf(:,:,i),macrosirf(:,:,i)] = CCMirf(Ppi,factorcoef,Phif,Q,nirf,nlength,nlag,nvollag);
    [finirf(:,:,i),finsirf(:,:,i)] = CCMirf(Ppi,factorcoef,Phif,Q,nirf2,nlength,nlag,nvollag);
    
end

%% This code calculates the sign restrictions
nlength = 40;
nlag    = 6;
nvollag = 2;
nirf    = 1;
nirf2   = 2;
isrobust = 1; %isrobust = 1 makes it the prior robust specification
ispostmean = 0; %ispostmean only looks at the posterior mean

KK = 1; %KK is the lenght of the restriciton


if ispostmean == 1
    Pimat         = mean(Pimat,3);
    factorcoefmat = mean(factorcoefmat,3);
    PhiFmat       = mean(PhiFmat,3);
    if isrobust == 1
        nQ = 10000;
        nsave = 1;
    elseif isrobust == 0
        nQ = 10000;
        nsave = 1;
    end
else
    if isrobust == 1
        nQ = 10000;
        
        nsave = 500;
    elseif isrobust == 0
        nQ = 100;
        nsave = 5000;
    end
end

big_irf = zeros(30,nlength,2,nsave*nQ);
big_sirf = zeros(2,nlength,2,nsave*nQ);

rest2a = zeros(2,nsave*nQ);
rest2b = zeros(2,nsave*nQ);
rest1  = zeros(2,nsave*nQ);

restmax = zeros(2,nsave*nQ);
restmax2 = zeros(2,nsave*nQ);

nskip = floor(ndraws/nsave);

for i = 1:nsave
    tic
    sind = (i-1)*nskip+1
    Ppi        = Pimat(:,:,sind);
    Ppi        = Ppi';
    factorcoef = factorcoefmat(:,:,sind);
    factorcoef = factorcoef';
    Phif       = PhiFmat(:,:,sind);
    
    tempirf = zeros(30,nlength,2,nQ);
    tempsirf = zeros(2,nlength,2,nQ);
    temprest2a = zeros(2,nQ);
    temprest2b = zeros(2,nQ);
    temprest1  = zeros(2,nQ);
    tempmax   = zeros(2,nQ); %This is storing the max spread response
    tempmax2   = zeros(2,nQ); %This is storing the max SP500 response
    for jj = 1:nQ
        %Drawing the Q
        [Q,RRR] = qr(randn(2)); %random rotation
        for i0=1:2;
            if RRR(i0,i0)<0 %normalization, See Zha's paper
                Q(:,i0)=-Q(:,i0);
            end
        end
        
        [tempirf(:,:,1,jj),tempsirf(:,:,1,jj)] = CCMirf(Ppi,factorcoef,Phif,Q,nirf,nlength,nlag,nvollag);
        [tempirf(:,:,2,jj),tempsirf(:,:,2,jj)] = CCMirf(Ppi,factorcoef,Phif,Q,nirf2,nlength,nlag,nvollag);
        
        if all(tempsirf(1,1:KK,1,jj)>0) == 1
            temprest2a(1,jj) = 1;
        end
        
        if all(tempsirf(1,1:KK,2,jj)>0) == 1
            temprest2a(2,jj) = 1;
        end
        
        if all(tempsirf(2,1:KK,1,jj)>0) == 1
            temprest2b(1,jj) = 1;
        end
        
        if all(tempsirf(2,1:KK,2,jj)>0) == 1
            temprest2b(2,jj) = 1;
        end
        
        if all(tempirf(20,1:KK,1,jj)>0) == 1
            temprest1(1,jj) = 1;
        end
        
        if all(tempirf(20,1:KK,2,jj)>0) == 1
            temprest1(2,jj) = 1;
        end
        
        %Storing things to figure out the max spread response
        if tempirf(20,1,1,jj) > tempirf(20,1,2,jj)
            tempmax(1,jj) = 1;
        else
            tempmax(2,jj) = 1;
        end
        
        %Storing things to figure out the max SP500 response
        if tempirf(19,1,1,jj) < tempirf(19,1,2,jj)
            tempmax2(1,jj) = 1;
        else
            tempmax2(2,jj) = 1;
        end
    end
    big_irf(:,:,:,(i-1)*nQ+1:i*nQ) = tempirf;
    big_sirf(:,:,:,(i-1)*nQ+1:i*nQ) = tempsirf;
    
    rest2a(:,(i-1)*nQ+1:i*nQ)  = temprest2a;
    rest2b(:,(i-1)*nQ+1:i*nQ)  = temprest2b;
    rest1(:,(i-1)*nQ+1:i*nQ)   = temprest1;
    restmax(:,(i-1)*nQ+1:i*nQ) = tempmax;
    restmax2(:,(i-1)*nQ+1:i*nQ) = tempmax2;
    toc
end


%% We plot here the robust sign restrictions IRF
if isrobust == 1
    labels= {'Employees','IP','CU: Mnaufacturing','Help wanted/employment ratio','UR','RPI','Weekly hrs: goods producing','Housing starts','Housing permits','Real cons spending','Real mfcting trade sales','ISM new orders index','Orders for durable goods','Avg hourly earnings, goods producing','PPI Finished Goods','PPI commodities','PCE','FEDFUNDS','SP500','BAAT10Y','Excess Return','SMB','HML','Momentum','R15_R11','aggind1','aggind2','aggind3','aggind4','aggind5'};
    transformmat=[1 1 1 1 1 1 0 0 0 1 1 0 1 2 2 2 2 1 1 0 0 0 0 0 0 0 0 0 0 0];
    
    % transformmat=[1 1 1 1 1 1 0 0 0 1 1 0 1 1 1 1 1 1 1 0 0 0 0 0 0 0 0 0 0 0];
    
    
    %Determining which IRFs satisfy the sign restrictions
    
    rest2shockmac = (rest2a==1)&~restmax;
    rest2shockfin = (rest2b==1)&restmax;
    resttemp = (sum(rest2shockmac+rest2shockfin,1) == 2); %Need to make sure that both shocks are in the same Q
    restfull = repmat(resttemp,2,1);
    restcomb = restfull&rest2shockfin;
    
    big_irf_collect_max = zeros(30,nlength,nsave);
    big_irf_collect_min = zeros(30,nlength,nsave);
    
    big_sirf_collect_max = zeros(2,nlength,nsave);
    big_sirf_collect_min = zeros(2,nlength,nsave);
    
    
    for i = 1:nsave
        i
        resttemp = restcomb(:,(i-1)*nQ+1:i*nQ);
        big_irftemp = big_irf(:,:,:,(i-1)*nQ+1:i*nQ);
        big_irftemp = big_irftemp(:,:,resttemp);
        big_sirftemp = big_sirf(:,:,:,(i-1)*nQ+1:i*nQ);
        big_sirftemp = big_sirftemp(:,:,resttemp);
        if size(big_irftemp,3) > 0
            for ntarget = 1:30
                if transformmat(ntarget) == 1
                    big_irf_collect_max(ntarget,:,i) = max(cumsum(big_irftemp(ntarget,:,:)),[],3);
                    big_irf_collect_min(ntarget,:,i) = min(cumsum(big_irftemp(ntarget,:,:)),[],3);
                elseif transformmat(ntarget) == 0
                    big_irf_collect_max(ntarget,:,i) = max(big_irftemp(ntarget,:,:),[],3);
                    big_irf_collect_min(ntarget,:,i) = min(big_irftemp(ntarget,:,:),[],3);
                elseif transformmat(ntarget) == 2
                    big_irf_collect_max(ntarget,:,i) = max(cumsum(cumsum(big_irftemp(ntarget,:,:))),[],3);
                    big_irf_collect_min(ntarget,:,i) = min(cumsum(cumsum(big_irftemp(ntarget,:,:))),[],3);
                end
            end
            for ntarget = 1:2
                big_sirf_collect_max(ntarget,:,i) = max(big_sirftemp(ntarget,:,:),[],3);
                big_sirf_collect_min(ntarget,:,i) = min(big_sirftemp(ntarget,:,:),[],3);
            end
        else
            big_irf_collect_max(:,:,i) = NaN;
            big_irf_collect_min(:,:,i) = NaN;
            big_sirf_collect_max(:,:,i) =NaN;
            big_sirf_collect_min(:,:,i) = NaN;
            
        end
    end
    
    %%
    labels= {'Employees','IP','CU: Mnaufacturing','Help wanted/employment ratio','UR','RPI','Weekly hrs: goods producing','Housing starts','Housing permits','Real cons spending','Real mfcting trade sales','ISM new orders index','Orders for durable goods','Avg hourly earnings, goods producing','PPI Finished Goods','PPI commodities','PCE','FEDFUNDS','SP500','BAAT10Y','Excess Return','SMB','HML','Momentum','R15_R11','aggind1','aggind2','aggind3','aggind4','aggind5'};
    
    scalefactor(1) = 0.2179; %Emp
    scalefactor(2) = 0.7562; %IP
    scalefactor(3) = 0.6521; %CU
    scalefactor(4) = 0.0624; %Help wanted/employment ratio
    scalefactor(5) = 0.1791; %Unemployment
    scalefactor(6) = 0.5499; %RPI
    scalefactor(7) = 0.6007;
    scalefactor(8) = 32.4357;
    scalefactor(9) = 32.1961;
    scalefactor(10) = 0.5262;
    scalefactor(11) = 1.0961;
    scalefactor(12) = 7.8065;
    scalefactor(13) = 3.7218;
    scalefactor(14) = 0.4060;
    scalefactor(15) = 0.6597;
    scalefactor(16) = 3.2505;
    scalefactor(17) = 0.1891;%PCE
    scalefactor(18) = 0.5407; %FFR
    scalefactor(19) = 3.6135; %SP500
    scalefactor(20) = 0.8162; %BAA
    scalefactor(21) = 4.4647;
    scalefactor(22) = 3.0709;
    scalefactor(23) = 2.8235;
    scalefactor(24) = 4.1714;
    scalefactor(25) = 4.0997;
    scalefactor(26) = 4.5605;
    scalefactor(27) = 4.3052;
    scalefactor(28) = 5.4612;
    scalefactor(29) = 4.9477;
    scalefactor(30) = 5.3015;
    
    xcal0 = 0:1:36;
    
    %% Giacomini and Kitagawa (2015) robust sets
    [~,nlength,~] = size(big_irf_collect_max);
    
    q1irf = zeros(nlength,1);
    q2irf = zeros(nlength,1);
    
    for jj = 20
        for i = 1:nlength
            big_irfv_min = scalefactor(jj)*squeeze(big_irf_collect_min(jj,i,:));
            big_irfv_max = scalefactor(jj)*squeeze(big_irf_collect_max(jj,i,:));
            
            robustcred2 = @(r) robustcred(r,big_irfv_min,big_irfv_max,0.7);
            rmin = fminsearch(robustcred2,0.1);
            zalphrmin = robustcred2(rmin);
            
            q1irf(i) = rmin-zalphrmin;
            q2irf(i) = rmin+zalphrmin;
        end
    end

pmbmax = mean(big_irf_collect_max,3);
pmbmin = mean(big_irf_collect_min,3);    
    
  [ph,msg]=jbfill([0:36],q1irf(1:37)',q2irf(1:37)', rgb('lightcoral'), rgb('lightcoral'), 1, 0.4);        
    hold on
    plot(zeros(37,1),'k--','linewidth',2)
    plot([0:36],scalefactor(jj)*pmbmin(jj,1:37)','color', rgb('firebrick'),'linewidth',2)    
    plot([0:36],scalefactor(jj)*pmbmax(jj,1:37)','color', rgb('firebrick'),'linewidth',2)
    xlabel('Months')
    ylabel('Percent')
    set(gca, 'XTick', [0:6:36])
    xlim([0 36])
%     ylim([-1 1])
        set(gca,'fontsize', 20, 'linewidth', 2)
    
    
end