%This code computes the IRFs for the levels and the volatilities of the
%CCM (2017) model
function [irf,sirf] = CCMirf(Ppi,factorcoef,Phif,Q,nirf,nlength,nlag,nvollag)
%We know that the lag length of the vol regressors in the VAR is the same as
%the lag length of the vol VAR (= 2 in CCM's specification)
[numobs,~] = size(Ppi);
[numfact,~] = size(factorcoef);

irf = zeros(numobs,nlength+nlag);
sirf = zeros(numfact,nlength+nvollag);

sirf(:,nvollag+1) = chol(Phif,'lower')*Q(:,nirf);
flipirf = fliplr(irf(:,1:nlag));
flipirf = flipirf';
flipsirf = fliplr(sirf(:,1:nvollag+1));
flipsirf = flipsirf';
xvar = [flipirf(:);flipsirf(:)];
irf(:,nlag+1)     =  Ppi*xvar;

for i = nlag+2:nlength+nlag
    %Updating the volatility equation
    flipsirf = fliplr(sirf(:,i-nlag:i-nlag+nvollag-1));
    flipsirf = flipsirf';
    xvarvol = [flipsirf(:);irf(:,i-1)];
    sirf(:,i-nlag+nvollag) = factorcoef*xvarvol;
    
    %Updating the level equation
    flipirf = fliplr(irf(:,i-nlag:i-1));
    flipirf = flipirf';
    flipsirf = fliplr(sirf(:,i-nlag:i-nlag+nvollag));
    flipsirf = flipsirf';
    xvar = [flipirf(:);flipsirf(:)];
    irf(:,i) = Ppi*xvar;
end

irf = irf(:,nlag+1:end);
sirf = sirf(:,nvollag+1:end);

end